from django.db import models
from django.http import HttpResponseRedirect
from wagtail.admin.panels import (
    TabbedInterface, ObjectList, FieldPanel, InlinePanel, MultiFieldPanel, FieldRowPanel, HelpPanel,
    PublishingPanel,
)
from wagtail.fields import StreamField, RichTextField
from wagtail.models import Page,  PreviewableMixin, Orderable, PageManager
from wagtail.search import index
from wagtail.snippets.models import register_snippet, register_deferred_snippets
from modelcluster.models import ClusterableModel
from modelcluster.fields import ParentalKey
from modelcluster.contrib.taggit import ClusterTaggableManager
from taggit.models import TagBase, GenericTaggedItemBase,ItemBase
from wagtail_trash.managers import TrashManager
from wagtail.blocks import *


from wagtailmodelchooser import register_model_chooser

from django import forms

from django_extensions.db.fields import AutoSlugField
from django.template.defaultfilters import slugify

import uuid

from django_ckeditor_5.fields import CKEditor5Field
from django_countries.fields import CountryField
from wagtail_color_panel.fields import ColorField
from wagtail_color_panel.edit_handlers import NativeColorPanel

from wagtail.contrib.settings.models import (
    BaseGenericSetting,
    BaseSiteSetting,
    register_setting,
)
# @register_snippet
from wagtail_color_panel.blocks import NativeColorBlock

from wagtailmodelchooser.blocks import ModelChooserBlock
from .common import *
from .blocks import *
from wagtail.blocks import *


@register_setting
class SiteThemeSettings(BaseSiteSetting):
    topBgColor = ColorField(default="#FFFFFF", verbose_name="BG")
    topFgColor = ColorField(default="#000000", verbose_name="FG")
    topHoverColor = ColorField(default="#000000", verbose_name="Hover")
    
    menuBgColor = ColorField(default="#FFFFFF", verbose_name="BG")
    menuFgColor = ColorField(default="#000000", verbose_name="FG")
    menuHoverBGColor = ColorField(default="#000000", verbose_name="Hover BG")
    menuHoverFGColor = ColorField(default="#000000", verbose_name="Hover FG")
    
    footerBgColor = ColorField(default="#000000", verbose_name="BG")
    footerFgColor = ColorField(default="#000000", verbose_name="FG")
    footerHoverColor = ColorField(default="#000000", verbose_name="Hover")
    
    arrowBgColor = ColorField(default="#000000", verbose_name="BG")
    arrowFgColor = ColorField(default="#000000", verbose_name="FG")
    arrowHoverColor = ColorField(default="#000000", verbose_name="Hover")
    
    bannerBgColor = ColorField(default="#000000", verbose_name="BG")
    bannerFgColor = ColorField(default="#000000", verbose_name="FG")
    
    dbTitleBgColor = ColorField(default="#000000", verbose_name="Title BG")
    dbTitleFgColor = ColorField(default="#000000", verbose_name="Title FG")
    dbBodyBgColor = ColorField(default="#000000", verbose_name="Body BG")
    dbBodyFgColor = ColorField(default="#000000", verbose_name="Body FG")
    
    tileBgColor = ColorField(default="#000000", verbose_name="BG")
    tileFgColor = ColorField(default="#000000", verbose_name="FG")
    tileHoverColor = ColorField(default="#000000", verbose_name="Hover")

    quoteBgColor = ColorField(default="#000000", verbose_name="BG")
    quoteFgColor = ColorField(default="#000000", verbose_name="FG")
    quoteHoverBgColor = ColorField(default="#000000", verbose_name="Hover BG")
    quoteHoverFgColor = ColorField(default="#000000", verbose_name="Hover FG")
    
    searchBgColor = ColorField(default="#000000", verbose_name="BG")
    searchFgColor = ColorField(default="#000000", verbose_name="FG")
    
    # Mega Menu
    menuListBgColor = ColorField(default="#f0f0f0", verbose_name="BG")
    menuListFgColor = ColorField(default="#000000", verbose_name="FG")
    menuListHoverBgColor = ColorField(default="#000000", verbose_name="Hover BG")
    menuListHoverFgColor = ColorField(default="#FFFFFF", verbose_name="Hover FG")
    
    menuImgTileBgColor = ColorField(default="#FFFFFF", verbose_name="BG")
    menuImgTileFgColor = ColorField(default="#000000", verbose_name="FG")
    menuImgTileHoverBgColor = ColorField(default="#000000", verbose_name="Hover BG")
    menuImgTileHoverFgColor = ColorField(default="#FFFFFF", verbose_name="Hover FG")
    
    
    panels = [
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        NativeColorPanel('menuListBgColor'),
                        NativeColorPanel('menuListFgColor'),
                        NativeColorPanel('menuListHoverBgColor'),
                        NativeColorPanel('menuListHoverFgColor'),
                    ],heading="Menu List"
                ),
                FieldRowPanel(
                    [
                        NativeColorPanel('menuImgTileBgColor'),
                        NativeColorPanel('menuImgTileFgColor'),
                        NativeColorPanel('menuImgTileHoverBgColor'),
                        NativeColorPanel('menuImgTileHoverFgColor'),
                    ],heading="Menu Image Tile"
                ),
            ],heading="Mega Menu"
        ),
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        NativeColorPanel('searchBgColor'),
                        NativeColorPanel('searchFgColor'),
                    ]
                ),
            ],heading="Search"
        ),
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        NativeColorPanel('quoteBgColor'),
                        NativeColorPanel('quoteFgColor'),
                        NativeColorPanel('quoteHoverBgColor'),
                        NativeColorPanel('quoteHoverFgColor'),
                    ]
                ),
            ],heading="Quote"
        ),
        
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        NativeColorPanel('topBgColor'),
                        NativeColorPanel('topFgColor'),
                        NativeColorPanel('topHoverColor'),
                    ]
                ),
            ],heading="Top Header"
        ),
        
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        NativeColorPanel('menuBgColor'),
                        NativeColorPanel('menuFgColor'),
                        NativeColorPanel('menuHoverBGColor'),
                        NativeColorPanel('menuHoverFGColor'),
                    ]
                ),
            ],heading="Menubar"
        ),
        
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        NativeColorPanel('footerBgColor'),
                        NativeColorPanel('footerFgColor'),
                        NativeColorPanel('footerHoverColor'),
                    ]
                ),
            ],heading="Footer"
        ),
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        NativeColorPanel('arrowBgColor'),
                        NativeColorPanel('arrowFgColor'),
                        NativeColorPanel('arrowHoverColor'),
                    ]
                ),
            ],heading="Bottom to Top Arrow"
        ),
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        NativeColorPanel('bannerBgColor'),
                        NativeColorPanel('bannerFgColor'),
                    ]
                ),
            ],heading="Banner"
        ),
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        NativeColorPanel('dbTitleBgColor'),
                        NativeColorPanel('dbTitleFgColor'),
                        NativeColorPanel('dbBodyBgColor'),
                        NativeColorPanel('dbBodyFgColor'),
                    ]
                ),
            ],heading="Dialog Box"
        ),
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        NativeColorPanel('tileBgColor'),
                        NativeColorPanel('tileFgColor'),
                        NativeColorPanel('tileHoverColor'),
                    ]
                ),
            ],heading="Tile Theme"
        ),
        
    ]


@register_setting
class CurrencySettings(BaseSiteSetting):
    currency = models.CharField(
        max_length=255, help_text='INR,USD,etc...')
    icon = models.CharField(
        max_length=255, help_text='Choosefrom boxicons')


class TopHeader(models.Model):
    title=models.CharField(max_length=200,unique=True,choices=[('Top Header','Top Header'),('Header','Header'),('Bottom Menu','Bottom Menu'),('Burger Menu','Burger Menu'),('Mobile Header','Mobile Header')])
    size = models.CharField(
        max_length=200, choices=FONT_SIZE_LIST, blank=True, null=True)
    content = StreamField([
        ('row', StructBlock([
            ('bgColor',NativeColorBlock(default="#FFFFFF",label="BG Color")),
            ('fgColor',NativeColorBlock(default="#000000",label="FG Color")),
            ('hoverBgColor',NativeColorBlock(default="#000000",label="Hover BG Color")),
            ('hoverFgColor',NativeColorBlock(default="#FFFFFF",label="Hover FG Color")),
            ('content',StreamBlock([
                ('column', StreamBlock([
                            ('company', ModelChooserBlock('core.CompanyInfo')),
                            ('social', StreamBlock([
                                ('item', ModelChooserBlock('core.Social')),
                            ])),
                            ('page',PageChooserBlock(required=False)),
                            ('show_login', BooleanBlock(required=False,default=False)),
                            ('show_search', BooleanBlock(required=False,default=False)),
                            ('show_burger_menu', BooleanBlock(required=False,default=False)),
                            ('show_news', IntegerBlock(required=False,default=5,label="News Scroll Time in seconds")),
                            ('mega_menu',StructBlock([
                                ('page',PageChooserBlock(required=False)),
                                # ('title', CharBlock(required=False)),
                                ('body',StreamBlock([
                                    ('col',StreamBlock([
                                        ("heading", HeadingBlock(required=False)),
                                        ("hr_line", HRLine(required=False)),
                                        ("line_break", LineBreak(required=False)),
                                        ('page',PageChooserBlock(required=False)),
                                        ('show_cat', BooleanBlock(required=False,default=False)),
                                        ('show_price', BooleanBlock(required=False,default=False)),
                                        ('show_feature', BooleanBlock(required=False,default=False)),
                                        ('show_brand', BooleanBlock(required=False,default=False)),
                                        ('option', StructBlock([
                                            ('image', ImageBlock(required=False)),
                                            ('page', PageChooserBlock(required=False)),
                                            ('text', CharBlock(required=False))
                                        ],label="Thumbnail"))
                                    ],required=False))
                                    
                                ],required=False))
                            ])),
                            ('accordion',StructBlock([
                                ('page',PageChooserBlock(required=False)),
                                ('content',StreamBlock([
                                    ('page',PageChooserBlock(required=False)),
                                ]))
                            ]))
                            
                        ], required=False
                    )
                ),
            ]))
            
        ])),
    ], blank=True, null=True, use_json_field=True)
    panels = [
        MultiFieldPanel(
            [

                FieldRowPanel(
                    [
                        FieldPanel('title'),
                        FieldPanel('size'),
                    ]
                ),
                FieldPanel('content', heading="Body")
            ]
        )
    ]
    def __str__(self):
        return str(self.title)
    class Meta:
        verbose_name="Header & Menu"

@register_setting
class BottomFooter(BaseSiteSetting):
    bgColor = ColorField(default="#FFFFFF", verbose_name="BG")
    fgColor = ColorField(default="#000000", verbose_name="FG")
    size = models.CharField(
        max_length=200, choices=FONT_SIZE_LIST, blank=True, null=True)
    content = StreamField([
        ('row', StreamBlock([
            ('column', StreamBlock(
                [
                    ('social', StreamBlock([
                        ('item', ModelChooserBlock('core.Social')),
                    ])),
                    ('company', ModelChooserBlock('core.CompanyInfo')),
                    ('heading', CharBlock()),
                    ('paragraph', RichTextBlock()),
                    ('link', StructBlock(
                        [
                            ('text', CharBlock()),
                            ('url', URLBlock()),

                        ]
                    )),
                ], required=False
            )),
        ])),
    ], blank=True, null=True, use_json_field=True)
    panels = [
        MultiFieldPanel(
            [

                FieldRowPanel(
                    [
                        NativeColorPanel('bgColor'),
                        NativeColorPanel('fgColor'),
                        FieldPanel('size'),
                    ]
                ),
                FieldPanel('content', heading="Body")
            ]
        )
    ]

class Footer(models.Model):
    title=models.CharField(default="Footer",max_length=200)
    body = StreamField([
        ('row', FooterRowBlock()),
    ], blank=True, null=True, use_json_field=True)
    panels = [
        FieldPanel('title'),
        FieldPanel('body'),
    ]

@register_snippet
class Subscriber(models.Model):
    email = models.EmailField(verbose_name="Email")
    subsDate = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.email

class Menu(ClusterableModel):
    name = models.CharField(
        max_length=255, verbose_name="Name", unique=True)
    slug = AutoSlugField(populate_from='name')

    def slugify_function(self, content):
        slug = content.replace(' ', '-').lower()
        return slug.replace('_', '-').lower()

    panels = [
        FieldPanel("name"),
        InlinePanel('menu_items', label="Sub Menu"),
    ]

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = 'Deals (Menu)'

class MenuItems(Orderable):
    menu = ParentalKey(Menu, related_name="menu_items")
    title = models.CharField('Title',
                             max_length=200, null=True, blank=True)
    slug = AutoSlugField(populate_from='title', editable=True)
    panels = [
        FieldPanel('title'),
        FieldPanel('slug')
    ]

    def slugify_function(self, content):
        slug = content.replace(' ', '-').lower()
        return content.replace('_', '-').lower()

    def __str__(self):
        return str(self.menu)+"    >   "+str(self.title)

@register_snippet
class CustomTag(TagBase):
    bgColor = ColorField(
        default="#FFFFFF", verbose_name="BG Color", blank=True, null=True)
    fgColor = ColorField(
        default="#000000", verbose_name="FG Color", blank=True, null=True)
    panels = [
        MultiFieldPanel([
            FieldRowPanel([

                FieldPanel('name'),
                FieldPanel('slug'),
            ]),
            FieldRowPanel([

                NativeColorPanel('bgColor'),
                NativeColorPanel('fgColor'),
            ])
        ])
    ]

    class Meta:
        verbose_name = "Tag"
        verbose_name_plural = "Tags"


@register_model_chooser
class CompanyInfo(models.Model):
    ciType = models.CharField(max_length=200, null=True, choices=[
        ('Name', 'Name'),
        ('Address', 'Address'),
        ('Email', 'Email'),
        ('Contact', 'Contact'),
        ('Logo', 'Logo')
    ], verbose_name="Info Type")
    ciName = models.CharField(
        max_length=200, null=True, blank=True, verbose_name="Name")
    ciTitle = models.CharField(
        max_length=200, null=True, blank=True, verbose_name="Title")
    ciLogo = models.ForeignKey(
        "wagtailimages.Image",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name="Logo"
    )
    ciLogoText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Logo Text")
    ciBody = StreamField([
        ('text', CharBlock()),
        ('paragraph', RichTextBlock()),
    ], blank=True, null=True, verbose_name="Body")

    # modify your content_panels:
    panels = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('ciType'),
                FieldPanel('ciName'),

                FieldPanel('ciTitle'),

            ]),
            FieldRowPanel([
                FieldPanel('ciLogo'),
                FieldPanel('ciLogoText'),
            ]),
        ]),
        FieldPanel('ciBody'),
    ]

    class Meta:
        verbose_name_plural = "Company Info"
        ordering = ['ciTitle']

    def __str__(self):
        if self.ciTitle and self.ciName:
            return f'{self.ciName}  ::  {self.ciTitle}  :: {self.ciType}'
        if self.ciTitle:
            return f'{self.ciTitle}  :: {self.ciType}'
        else:
            return f'{self.ciType}'


class PolicyType(ClusterableModel):
    plcType = models.CharField(
        max_length=255, verbose_name="Policy Type", unique=True)
    slug = AutoSlugField(populate_from='plcType')

    def slugify_function(self, content):
        slug = content.replace(' ', '-').lower()
        return slug.replace('_', '-').lower()
    panels = [
        FieldPanel("plcType"),
    ]

    def __str__(self):
        return self.plcType

    class Meta:
        verbose_name_plural = 'Policy Type'


@register_model_chooser
class ComPolicy(models.Model):
    type = models.ForeignKey(
        PolicyType, on_delete=models.SET_NULL, null=True, related_name="policies")
    plcCountry = CountryField(verbose_name="Country")
    title = models.CharField('Country Policy Title',
                             max_length=200, null=True, blank=True)
    slug = AutoSlugField(populate_from='title', editable=True)

    text = CKEditor5Field('Content', config_name='extends')
    created_on = models.DateTimeField(auto_now_add=True)
    updated_on = models.DateTimeField(auto_now=True)
    panels = [
        MultiFieldPanel([
            FieldRowPanel(
                [
                    FieldPanel('type'),
                    FieldPanel('plcCountry')
                ]
            ),
            FieldRowPanel(
                [
                    FieldPanel('title'),
                    # FieldPanel('slug')
                ]
            ),
            FieldPanel('text')
        ])
    ]

    def slugify_function(self, content):
        slug = content.replace(' ', '-').lower()
        return content.replace('_', '-').lower()

    def __str__(self):
        return str(self.type)+"    >   "+str(self.plcCountry)+"    >   "+str(self.title)

    def get_absolute_url(self):
        return f'/{self.slug}/'

    class Meta:
        verbose_name_plural = "Company Policy"


class SocialType(ClusterableModel):
    scType = models.CharField(
        max_length=255, verbose_name="Social Type", unique=True)
    panels = [
        FieldPanel("scType"),
        HelpPanel(
            "Select Enter the platform such as WhatsApp,Telegram in Messaging"),
        InlinePanel('social_platforms', label="Platform"),
    ]
    web_panels = [
        InlinePanel('web_socials', label="Web Social Link")
    ]
    edit_handler = TabbedInterface([
        ObjectList(panels, heading='Type & Platform'),
        ObjectList(web_panels, heading='Web Social'),

    ])

    def __str__(self):
        return self.scType

    class Meta:
        verbose_name_plural = 'Social Type'
        db_table = "common_scType"

@register_model_chooser
class Social(Orderable):
    type = ParentalKey(SocialType, related_name="web_socials")

    soTitle = models.CharField(blank=True, null=True,
                               max_length=255,
                               help_text="Social Plateform Name e.g: Facebook,Instagram",
                               verbose_name="Title"
                               )
    soUserId = models.CharField(blank=True, null=True,
                                max_length=255,
                                help_text="Social User Id",
                                verbose_name="User Id"
                                )

    TextfgColor = ColorField(
        default="#000000", verbose_name="FG", blank=True, null=True)
    bgColor = ColorField(
        default="#FFFFFF", verbose_name="BG", blank=True, null=True)
    fgColor = ColorField(
        default="#000000", verbose_name="FG", blank=True, null=True)

    hoverBgColor = ColorField(
        default="#FFFFFF", verbose_name="Hover BG", blank=True, null=True)
    hoverFgColor = ColorField(
        default="#000000", verbose_name="Hover FG", blank=True, null=True)

    soLogo = models.ForeignKey(
        "wagtailimages.Image",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        verbose_name="Icon"
    )
    soLogoText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Logo Text")

    soUrl = models.CharField(
        null=True,
        verbose_name="URL",
        max_length=255,
        help_text="For Whatsapp use https://wa.me/8859XXXXXX",
    )
    panels = [
        MultiFieldPanel([
            FieldRowPanel(
                [
                    FieldPanel("type"),
                    FieldPanel("soTitle"),

                ]
            ),
            FieldRowPanel(
                [
                    FieldPanel("soUserId"),
                    NativeColorPanel("TextfgColor")

                ]
            ),
            FieldRowPanel(
                [
                    FieldPanel("soLogo"),
                    FieldPanel("soLogoText"),
                ]),
            FieldRowPanel(
                [
                    FieldPanel("soUrl"),

                ]
            ),
        ]),
        MultiFieldPanel([
            FieldRowPanel([
                NativeColorPanel('bgColor'),
                NativeColorPanel('fgColor'),
            ]),
        ], heading="Active Color"),
        MultiFieldPanel([
            FieldRowPanel([
                NativeColorPanel('hoverBgColor'),
                NativeColorPanel('hoverFgColor'),
            ]),
            # FieldPanel('tags'),
        ], heading="Hover Color"),
    ]

    def __str__(self):
        return f'{self.soTitle}'

    class Meta:
        verbose_name = "Web Social Link"
        verbose_name_plural = "Web Social Links"
        ordering = ['soTitle']

@register_model_chooser
class SocialPlatform(Orderable):
    type = ParentalKey(SocialType, related_name="social_platforms")
    title = models.CharField(max_length=100, verbose_name="Name")
    logo = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.CASCADE, related_name='+', blank=True, null=True, verbose_name="Icon Image"
    )
    soLogoText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Icon Text")
    soPlURL=models.URLField(blank=True,null=True,verbose_name="Share URL")
    iconColor = ColorField(
        default="#FFFFFF", verbose_name="Icon Color", blank=True, null=True)
    iconHoverColor = ColorField(
        default="#000000", verbose_name="Icon Hover Color", blank=True, null=True)
    panels = [
        MultiFieldPanel(
            [
                FieldRowPanel(
                    [
                        FieldPanel("title", widget=forms.TextInput(
                            attrs={'placeholder': "Enter Platform Name e.g: Amazon,Flipkart,etc..."})),
                    ]
                ),
                FieldPanel('soLogoText'),
                FieldRowPanel(
                    [
                        FieldPanel('logo'),
                        NativeColorPanel('iconColor'),
                        NativeColorPanel('iconHoverColor'),
                    ]
                )
            ]
        ),
    ]

    def __str__(self):
        return str(self.type)+"    >   "+str(self.title)

    class Meta:
        verbose_name = "Social Platform"
        db_table = "common_socialPlatform"



class Enquiry(models.Model):
    id = models.UUIDField(unique=True, primary_key=True, editable=False,
                          default=uuid.uuid4, verbose_name='Public identifier')
    enqName = models.CharField(max_length=100, null=True, blank=True)
    enqEmail = models.EmailField(null=True, blank=True)
    enqContact = models.CharField(max_length=20, null=True, blank=True)
    enqCompany = models.CharField(max_length=100, null=True, blank=True)
    enqMessage = models.TextField(null=True, blank=True)
    enqCreatedOn = models.DateTimeField(auto_now_add=True)
    enqStatus = models.CharField(
        max_length=100, null=True, blank=True, default="Pending")
    parent = models.ForeignKey(
        'self', on_delete=models.CASCADE, null=True, blank=True, related_name='replies')
    # enqProduct = models.ForeignKey(
    #     Products, on_delete=models.CASCADE, null=True, blank=True)
    enqProduct = models.CharField(max_length=200, null=True, blank=True)

    def __str__(self):
        if self.parent == None:
            return f'{self.enqName}'
        else:
            return f'{self.parent}  => {self.enqName}'

    class Meta:
        ordering = ['-enqCreatedOn']

class FollowUp(models.Model):
    id = models.UUIDField(unique=True, primary_key=True, editable=False,
                          default=uuid.uuid4, verbose_name='Public identifier')
    enquiry = models.ForeignKey(
        Enquiry, on_delete=models.CASCADE, null=True, blank=True)
    detail = RichTextField(null=True, blank=True)
    assignTo = models.CharField(max_length=200, null=True, blank=True)
    remarks = models.TextField(null=True, blank=True)
    enqCreatedOn = models.DateTimeField(auto_now_add=True)
    parent = models.ForeignKey(
        'self', on_delete=models.CASCADE, null=True, blank=True, related_name='replie_follow')

    def __str__(self):
        if self.parent == None:
            return f'{self.assignTo}'
        else:
            return f'{self.enquiry}  => {self.parent} => {self.assignTo}'

    class Meta:
        ordering = ['-enqCreatedOn']


